"""

    Module: $RCSfile: RequestTest.py,v $
    Author: $Author: stevray $
    Date:   $Date: 2002/02/13 18:24:08 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.2 $'[11:-2]


True  = 1
False = 0

class request:
    """
        An object instantiated from this class emulates
        the Zope REQUEST object.  We can thus use such an
        object for testing (some) Zopish python scripts
        without having to import the entire Zope framework.
    """
    def __init__(self):
        self.clear()

    def keys(self):
        return self.__envVariables.keys()       \
             + self.__otherVariables.keys()     \
             + self.form.keys()                 \
             + self.cookies.keys()

    def values(self):
        return self.__envVariables.values()     \
             + self.__otherVariables.values()   \
             + self.form.values()               \
             + self.cookies.values()

    def items(self):
        return self.__envVariables.items()      \
             + self.__otherVariables.items()    \
             + self.form.items()                \
             + self.cookies.items()

    def has_key(self, key):
        if self.__otherVariables.has_key(key):
            return True
        elif self.form.has_key(key):
            return True
        elif self.cookies.has_key(key):
            return True
        else:
            return False

    def get(self, key, defaultValue=None):
        try:
            return self.__getitem__(key)
        except KeyError:
            return defaultValue

    def clear(self):
        self.__envVariables = {}
        self.__otherVariables = {}
        self.form = {}
        self.cookies = {}

    def copy(self):
        tmp = request()
        tmp.__envVariables = self.__envVariables.copy()
        tmp.__otherVariables = self.__otherVariables.copy()
        tmp.form = self.form.copy()
        tmp.cookies = self.cookies.copy()
        return tmp

    def set(self, key, value):
        self.__otherVariables[key] = value

    def __contains__(self, key):
        if self.__envVariables.has_key(key):
            return True
        elif self.__otherVariables.has_key(key):
            return True
        elif self.form.has_key(key):
            return True
        elif self.cookies.has_key(key):
            return True
        else:
            return False

    def __len__(self):
        return len(self.__envVariables)     \
             + len(self.__otherVariables)   \
             + len(self.form)               \
             + len(self.cookies)

    def __getitem__(self, key):
        if self.__envVariables.has_key(key):
            return self.__envVariables[key]
        elif self.__otherVariables.has_key(key):
            return self.__otherVariables[key]
        elif self.form.has_key(key):
            return self.form[key]
        elif self.cookies.has_key(key):
            return self.cookies[key]
        else:
            raise KeyError

    def __setitem__(self, key, value):
        self.__otherVariables[key] = value

    def __delitem__(self, key):
        if self.__envVariables.has_key(key):
            del self.__envVariables[key]
        elif self.__otherVariables.has_key(key):
            del self.__otherVariables[key]
        elif self.form.has_key(key):
            del self.form[key]
        elif self.cookies.has_key(key):
            del self.cookies[key]
        else:
            raise KeyError