"""

    Module: $RCSfile: TestAnonymousUser.py,v $
    Author: $Author: payne $
    Date:   $Date: 2002/06/25 22:09:37 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.18 $'[11:-2]

import sys
sys.path.insert(0,'..')

from TestDirectory import getTestDirectory
from Connection import *
from LDAPUtilities import _ldap

from string import index
import unittest

class testAnonymousUser(unittest.TestCase):

    """ Tests the AnonymousUser class in the Connection module. """

    def setUp(self):
        self.nobody = AnonymousUser(getTestDirectory())

    def test01(self):
        """ 01 testAnonymousUser: Anonymous bind test """
        self.failUnless(self.nobody)

    def test02(self):
        """ 02 testAnonymousUser: getDNfromUserName for valid user uid"""
        dn = self.nobody.getDNfromUserName('usera', loginAttr='uid')
        self.failUnless(dn.lower() == 'uid=usera,ou=people,o=test'.lower())

    def test03(self):
        """ 03 testAnonymousUser: getDNfromUserName for valid user cn"""
        dn = self.nobody.getDNfromUserName('User, Anne', loginAttr='cn')
        self.failUnless(dn.lower() == 'uid=usera,ou=people,o=test'.lower())

    def test04(self):
        """ 04 testAnonymousUser: getDNfromUserName for invalid user """
        self.assertRaises(NO_SUCH_OBJECT, self.nobody.getDNfromUserName, 'nonexistentuser')

    def test05(self):
        """ 05 testAnonymousUser: unbind test """
        self.nobody.unbind()
        self.assertRaises(NO_SUCH_OBJECT, self.nobody.getDNfromUserName, 'uid=usera,ou=people,o=test')

    def test06(self):
        """ 06 testAnonymousUser: test getChildGroups """
        testGroups = [MappingObject('cn=Directory Administrators,ou=Groups,o=test'),
                      MappingObject('cn=Divisional Administrators,ou=Groups,o=test'),
                      MappingObject('cn=Empty Group,ou=Groups,o=test'),
                      MappingObject('cn=Manager,ou=Groups,o=test'),
                      MappingObject('cn=System Administrators,ou=Groups,o=test'),
                      MappingObject('cn=Test Group,ou=Groups,o=test')]
        groups = self.nobody.getChildGroups("ou=Groups,o=test")
        self.failUnless( len(groups) == len(testGroups) )
        indexes = range(len(groups))
        for i in indexes:
            t1 = testGroups[i]
            t2 = groups[i]
            self.failUnless(t1.mappingName.lower() == t2.mappingName.lower())
            self.failUnless(t1.mappingDN.lower() ==\
                            t2.mappingDN.lower() )

    def test07(self):
        """ 07 testAnonymousUser: test getMatchingGroups """
        groups = self.nobody.getMatchingGroups("Manager")
        self.failUnless( len(groups) == 1 )
        self.failUnless( groups[0] == 'cn=Manager,ou=Groups,o=test' )

    def test08(self):
        """ 08 testAnonymousUser: test getChildOrgUnits """
        testOUs = [MappingObject( 'ou=Groups,o=test' ),
                   MappingObject( 'ou=People,o=test' ),
                   MappingObject( 'ou=Permanent Test OU,o=test' ),
                   MappingObject( 'ou=ST ScI,o=test' ) ]
        orgUnits = self.nobody.getChildOrgUnits("o=test")
        self.failUnless( len(orgUnits) == len(testOUs) )
        indexes = range(len(orgUnits))
        for i in indexes:
            t1 = testOUs[i]
            t2 = orgUnits[i]
            self.failUnless(t1.mappingName.lower() == t2.mappingName.lower())
            self.failUnless(t1.mappingDN.lower() ==\
                            t2.mappingDN.lower() )

    def test09(self):
        """ 09 testAnonymousUser: test whatKindOfGroup """
        group = "cn=Manager,ou=Groups,o=test"
        kind = self.nobody.whatKindOfGroup(group)
        self.failUnless( kind == 'groupOfUniqueNames' )

    def test10(self):
        """ 10 testAnonymousUser: is not allowed to ... """
        # read a password
        s = self.nobody.search('uid=usera,ou=People,o=test',
                               _ldap.SCOPE_BASE,
                               '(objectclass=*)',
                               ["objectClass","userPassword"])[0][1]
        self.failUnless( s.has_key('objectClass') )
        self.failIf( s.has_key('userPassword') )

        # write a password
        modlist = ( (_ldap.MOD_REPLACE,'userPassword','junk'),)
        self.failUnlessRaises( _ldap.INSUFFICIENT_ACCESS,
                               self.nobody.Connection.modify_s,
                               'uid=usera,ou=People,o=test',
                               modlist)

    def test11(self):
        """ 11 testAnonymousUser: testing orgUnitMemberBrowser """
        request = { 'mappingDN' : 'ou=ST ScI,o=test',
                    'user' : self.nobody, 
                    'tree-colspan' : 2,
                    'tree-level' : 0,
                    'URL0' : 'http://bogus' }
 
        output = self.nobody.orgUnitMemberBrowser( None, request )
        self.failUnless( index( output, 
                                'href="http://bogus?query=uid=usera"') > 0 )

    def test12(self):
        """ 12 testAnonymousUser: testing orgUnitMemberBrowser by dn"""
        request = { 'mappingDN' : 'ou=ST ScI,o=test',
                    'user' : self.nobody, 
                    'tree-colspan' : 2,
                    'tree-level' : 0,
                    'URL0' : 'http://bogus' }
 
        schema = self.nobody.LDAPDirectory.getPersonSchema()
        schema.setPersonsOUAttr('dn')
        output = self.nobody.orgUnitMemberBrowser( None, request )
        self.failUnless( index( output, 
                                'href="http://bogus?query=uid=abogus"') > 0 )

    def test13(self):
        """ 13 testAnonymousUser: testing orgUnitMemberBrowser private"""
        request = { 'mappingDN' : 'ou=ST ScI,o=test',
                    'user' : self.nobody, 
                    'tree-colspan' : 2,
                    'tree-level' : 0,
                    'URL0' : 'http://bogus',
                    'private' : 1 }
 
        output = self.nobody.orgUnitMemberBrowser( None, request )
        self.failUnless( index( output, 
                 'href="http://bogus?distinguishedName=uid%3Dabogus') > 0 )

    def test14(self):
        """ 14 testAnonymousUser: testing orgUnitMemberBrowser private by dn"""
        request = { 'mappingDN' : 'ou=ST ScI,o=test',
                    'user' : self.nobody, 
                    'tree-colspan' : 2,
                    'tree-level' : 0,
                    'URL0' : 'http://bogus',
                    'private' : 1 }
 
        schema = self.nobody.LDAPDirectory.getPersonSchema()
        schema.setPersonsOUAttr('dn')
        output = self.nobody.orgUnitMemberBrowser( None, request )
        self.failUnless( index( output, 
                 'href="http://bogus?distinguishedName=uid%3Dabogus') > 0 )

if __name__ == '__main__':
    
    tests = unittest.makeSuite(testAnonymousUser, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
