"""

    Module: $RCSfile: TestManagedSchema.py,v $
    Author: $Author: payne $
    Date:   $Date: 2002/06/28 20:16:07 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.26 $'[11:-2]

import sys
sys.path.insert(0,'..')

import unittest

from Connection import AuthenticatedUser
from LDAPExceptions import *
from LDAPUtilities import *
from ManagedSchema import ManagedSchema
from SchemaDefaults_test import *
from TestDirectory import getTestDirectory

class testManagedSchema(unittest.TestCase):

    def setUp(self):
        directory = getTestDirectory()
        self.user = AuthenticatedUser('testadmin', 'testadmin',
                       directory)
        self.schema = ManagedSchema( defaultUserSchema, 
                                     defaultUserDisplayCategories,
                                     defaultUserNamingAttribute,
                                     defaultUserControllers,
                                     defaultUserSearchFilter,
                                     defaultUserSearchKeys,
                                     defaultUserSearchSortKey)

    def test01(self):
        """ 01 testManagedSchema: testing object creation """
        self.failUnless( listsAreEquivalent( self.schema.getDisplayCategories(),
                                             defaultUserDisplayCategories) )

    def test02(self):
        """ 02 testManagedSchema: testing getAllowedAttributeNames """
        allowed = [
            'businessCategory', 'carLicense', 'cn', 'departmentNumber', 
            'description', 'displayName', 'employeeType',
            'facsimileTelephoneNumber', 'givenName', 'homePhone',
            'homePostalAddress', 'initials', 'l', 'mail',
            'mailAlternateAddress', 'mailHost', 'mailRoutingAddress', 
            'manager', 'mobile', 'o', 'objectClass', 'ou', 'pager', 
            'physicalDeliveryOfficeName', 'postOfficeBox', 'postalAddress',
            'postalCode', 'roomNumber', 'secretary', 'seeAlso', 'sn', 'st', 
            'street', 'telephoneNumber', 'title', 'uid', 'userPassword', ]

        found = self.schema.getAllowedAttributeNames()
        for item in allowed:
            self.failUnless( item in found )
            found.remove( item )
        self.failUnless( found == [])

    def test03(self):
        """ 03 testManagedSchema: testing getAttributeCollection """
        found = self.schema.getAttributeCollection()
        self.failUnless( listsAreEquivalent( 
                         self.schema.getAllowedAttributeNames(), found.keys()))

    def test04(self):
        """ 04 testManagedSchema: testing getAttributeSchema """
        schema = self.schema.getAttributeSchema( 'cn' )
        self.failUnless( schema.getRequired() )

        self.assertRaises(AssertionError,
                          self.schema.getAttributeSchema,
                          'badValue')

    def test05(self):
        """ 05 testManagedSchema: testing getAvailableAttributeNames """
        self.failUnless( self.schema.getAvailableAttributeNames() == [] )

    def test06(self):
        """ 06 testManagedSchema: testing getDisplayCategories """
        self.failUnless( listsAreEquivalent( defaultUserDisplayCategories,
                         self.schema.getDisplayCategories() ) )

    def test07(self):
        """ 07 testManagedSchema: testing getManagedAttributeNames """
        managed = [
            'businessCategory', 'carLicense', 'cn', 'departmentNumber',
            'description', 'displayName', 'employeeType',
            'facsimileTelephoneNumber', 'givenName', 'homePhone',
            'homePostalAddress', 'initials', 'l', 'mail', 
            'mailAlternateAddress', 'mailHost', 'mailRoutingAddress', 
            'manager', 'mobile', 'o', 'objectClass', 'ou', 'pager', 
            'physicalDeliveryOfficeName', 'postOfficeBox', 'postalAddress', 
            'postalCode', 'roomNumber', 'secretary', 'seeAlso', 'sn', 'st', 
            'street', 'telephoneNumber', 'title', 'uid', 'userPassword']

        found = self.schema.getManagedAttributeNames()
        for item in managed:
            self.failUnless( item in found )
            found.remove( item )
        self.failUnless( found == [] )

    def test08(self):
        """ 08 testManagedSchema: testing getManagedNotRequiredAttributeNames """
        managedNR = [
            'businessCategory', 'carLicense', 'departmentNumber',
            'description', 'displayName', 'employeeType',
            'facsimileTelephoneNumber', 'givenName', 'homePhone',
            'homePostalAddress', 'initials', 'l', 'mail',
            'mailAlternateAddress', 'mailHost', 'mailRoutingAddress',
            'manager', 'mobile', 'o', 'ou', 'pager',
            'physicalDeliveryOfficeName', 'postOfficeBox', 'postalAddress',
            'postalCode', 'roomNumber', 'secretary', 'seeAlso', 'st',
            'street', 'telephoneNumber', 'title', 'userPassword',]
        found = self.schema.getManagedNotRequiredAttributeNames()
        for item in managedNR:
            self.failUnless( item in found )
            found.remove( item )
        self.failUnless( found == [] )
        
    def test09(self):
        """ 09 testManagedSchema: testing getPointsToTargets """
        self.failUnless( listsAreEquivalent( defaultPointsToTargets,
                                             self.schema.getPointsToTargets() ))

    def test10(self):
        """ 10 testManagedSchema: testing getRequiredAttributeNames """
        required = ['cn', 'objectClass', 'sn', 'uid']
        found = self.schema.getRequiredAttributeNames()
        for item in required:
            self.failUnless( item in found )
            found.remove( item )
        self.failUnless( found == [] )

    def test11(self):
        """ 11 testManagedSchema: testing getNamingAttribute """
        self.failUnless( \
            defaultUserNamingAttribute == self.schema.getNamingAttribute())

    def test12(self):
        """ 12 TestManagedSchema: testing add/del/set DisplayCategory """
        categories = defaultUserDisplayCategories
        categories.append( 'new category' )
        self.schema.addDisplayCategory( 'new category' )
        self.failUnless( listsAreEquivalent( categories,
                                        self.schema.getDisplayCategories() ))

        categories.remove( 'new category' )
        self.schema.delDisplayCategory( 'new category' )
        self.failUnless( listsAreEquivalent( categories,
                                        self.schema.getDisplayCategories() ))

        self.assertRaises(AssertionError,
                          self.schema.delDisplayCategory,
                          'badValue')

        categories = ['a', 'b', 'c' ]
        self.schema.setDisplayCategories( categories )
        self.failUnless( listsAreEquivalent( categories,
                                        self.schema.getDisplayCategories() ))

        categories = ['a',]
        self.schema.setDisplayCategories( 'a' )
        self.failUnless( listsAreEquivalent( categories,
                                        self.schema.getDisplayCategories() ))


    def test13(self):
        """ 13 TestManagedSchema: testing unmanageAttributeSchema """
        managed = self.schema.getManagedAttributeNames()
        managed.remove( 'businessCategory' )
        self.schema.unmanageAttributeSchema( 'businessCategory' )
        self.failUnless( listsAreEquivalent( managed,
                         self.schema.getManagedAttributeNames() ))

        self.failUnlessRaises(REQUIRED_ATTRIBUTE,
                              self.schema.unmanageAttributeSchema,
                              'cn')

        self.assertRaises(AssertionError,
                          self.schema.unmanageAttributeSchema,
                          'badValue')

    def test14(self):
        """ 14 TestManagedSchema: testing setNamingAttribute """
        self.schema.setNamingAttribute('cn')
        self.failUnless('cn' == self.schema.getNamingAttribute())

    def test15(self):
        """ 15 TestManagedSchema: testing setNamingAttribute with a bogus attribute """
        self.assertRaises(NO_SUCH_ATTRIBUTE, self.schema.setNamingAttribute, 'bogus')

    def test16(self):
        """ 16 TestManagedSchema: testing the getControllers function """
        testControllers = ['System Administrators']
        self.failUnless( listsAreEquivalent( self.schema.getControllers(),
                                             testControllers) )

    def test17(self):
        """ 17 TestManagedSchema: testing the add/delController functions """
        testCon1 = self.schema.getControllers()
        testCon2 = listCopy(testCon1)
        testCon2.append('newRole')
        self.schema.addController( 'newRole')
        self.failUnless( listsAreEquivalent( self.schema.getControllers(),
                                             testCon2) )
        self.schema.delController( 'newRole' )
        self.failUnless( listsAreEquivalent( self.schema.getControllers(),
                                             testCon1) )
        
    def test18(self):
        """ 18 TestManagedSchema: testing get/setSearchFilter functions """
        filter = self.schema.getSearchFilter()
        self.failUnless( filter == '(objectclass=person)' )
        self.schema.setSearchFilter( '(a=b)' )
        filter = self.schema.getSearchFilter()
        self.failUnless( filter == '(a=b)' )

    def test19(self):
        """ 19 TestManagedSchema: testing get/setSearchKeys functions """
        peopleKeys = ['uid']
        peopleKeyNames = ['Username']
        self.failUnless( 
            listsAreEquivalent( self.schema.getSearchKeys(),
                                peopleKeys ) )
        self.failUnless( 
            listsAreEquivalent( self.schema.getSearchKeyNames(peopleKeys),
                                peopleKeyNames ) )
        peopleKeys = ['displayName','uid']
        peopleKeyNames = ['Display Name','Username']
        self.schema.setSearchKeys( peopleKeys )
        self.schema.setSearchKeyName( 'displayName', 'Display Name' )
        self.failUnless( 
            listsAreEquivalent( self.schema.getSearchKeys(),
                                peopleKeys ) )
        self.failUnless( 
            listsAreEquivalent( self.schema.getSearchKeyNames(peopleKeys),
                                peopleKeyNames ) )

    def test20(self):
        """ 20 TestManagedSchema: testing get/setSearchSortKey functions """
        self.failUnless( self.schema.getSearchSortKey() == 'uid' )
        self.schema.setSearchSortKey( 'displayName' )
        self.failUnless( self.schema.getSearchSortKey() == 'displayName' )

    def test21(self):
        """ 21 TestManagedSchema: testing getRead/WriteableAttributesByRole """
        readable = self.schema.getReadableAttributesByRole('SELF')
        self.failUnless( 'objectClass' in readable )
        writeable = self.schema.getWriteableAttributesByRole('SELF')
        self.failUnless( 'userPassword' in writeable )

if __name__ == '__main__':
    
    tests = unittest.makeSuite(testManagedSchema, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
