"""

    Module: $RCSfile: TestSearchLister.py,v $
    Author: $Author: payne $
    Date:   $Date: 2002/06/25 22:09:37 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.2 $'[11:-2]

import sys
sys.path.insert(0,'..')

from Connection import *
from LDAPUtilities import _ldap
from LDAPUtilities import *
from SearchLister import SearchLister
from TestDirectory import getTestDirectory
import unittest

class testSearchLister(unittest.TestCase):
    """ Tests the SearchLister class """

    def setUp(self):
        self.directory = getTestDirectory()
        self.user = AnonymousUser(self.directory)
#        self.user = AuthenticatedUser('testadmin', 'testadmin',
#                       self.directory)

    def test01(self):
        """01 testSearchLister: creating object """
        schema = self.directory.getPersonSchema()
        searchLister = SearchLister( self.user, 'user', 'uid', 'person',
                                     self.directory.getPersonSearchBase(), 
                                     None, None, None, schema ).resultsList
        self.failUnless(searchLister)

    def test02(self):
        """ 02 testSearchLister: sort by uid """
        schema = self.directory.getPersonSchema()
        searchLister = SearchLister(self.user, 'user', 'uid', 'person',
                                    self.directory.getPersonSearchBase(), 
                                    None, None, None, schema ).resultsList
        self.failUnless(searchLister[0]['uid'][0].lower() == 'usera')
        self.failUnless(searchLister[0]['distinguishedName'][0].lower() == \
                        'uid=usera,ou=people,o=test')

    def test03(self):
        """ 03 testSearchLister: searching by ou """
        schema = self.directory.getPersonSchema()
        searchLister = SearchLister(self.user, 'ou=ST ScI,o=test', 'uid',
                                    'person',
                                    self.directory.getPersonSearchBase(), 
                                    None, None, None, schema ).resultsList
        self.failUnless( len(searchLister) == 3 )
        uids = []
        for row in searchLister:
            uids.append( row['uid'][0].lower() )
        for uid in ['usera','abogus','bbogus']:
            self.failUnless( uid in uids )

    def test04(self):
        """ 04 testSearchLister: searching by namingAttribute """
        schema = self.directory.getPersonSchema()
        searchLister = SearchLister(self.user, 'uid=usera', 'uid', 'person',
                                    self.directory.getPersonSearchBase(), 
                                    None, None, None, schema ).resultsList
        self.failUnless(searchLister[0]['uid'][0].lower() == 'usera')
        self.failUnless(searchLister[0]['distinguishedName'][0].lower() == 'uid=usera,ou=people,o=test')

    def test05(self):
        """ 05 testSearchLister: searching with people in OUs """
        schema = self.directory.getPersonSchema()
        schema.setPersonsOUAttr('dn')
        searchLister = SearchLister(self.user, 'bogus', 'uid', 'person',
                                    self.directory.getPersonSearchBase(), 
                                    None, None, None, schema ).resultsList
        self.failUnless(searchLister[0]['uid'][0].lower() == 'abogus')
        self.failUnless(searchLister[0]['distinguishedName'][0] == 'uid=abogus,ou=ST ScI,o=test')

    def test06(self):
        """ 06 testSearchLister: searching with people in OUs by namingAttribute"""
        schema = self.directory.getPersonSchema()
        schema.setPersonsOUAttr('dn')
        searchLister = SearchLister(self.user, 'uid=abogus', 'uid', 'person',
                                    self.directory.getPersonSearchBase(), 
                                    None, None, None, schema ).resultsList
        self.failUnless(searchLister[0]['uid'][0].lower() == 'abogus')
        self.failUnless(searchLister[0]['distinguishedName'][0] == 'uid=abogus,ou=ST ScI,o=test')

    def test07(self):
        """ 07 testSearchLister: searching with people in OUs"""
        schema = self.directory.getPersonSchema()
        schema.setPersonsOUAttr('dn')
        searchLister = SearchLister(self.user, 'bogus', 'uid', 
                                    'person', 'ou=ST ScI,o=test',
                                    None, schema.getSearchFilter(), 
                                    None, schema ).resultsList
        self.failUnless(searchLister[0]['uid'][0].lower() == 'abogus')
        self.failUnless(searchLister[0]['distinguishedName'][0] == 'uid=abogus,ou=ST ScI,o=test')
        self.failUnless(searchLister[1]['uid'][0].lower() == 'bbogus')
        self.failUnless(searchLister[1]['distinguishedName'][0] == 'uid=bbogus,ou=DestinationOU,ou=ST ScI,o=test')

    def test08(self):
        """ 08 testSearchLister: searching with people in OUs"""
        schema = self.directory.getPersonSchema()
        schema.setPersonsOUAttr('dn')
        searchLister = SearchLister(self.user, 'bogus', 'uid', 
                                    'person', 'ou=ST ScI,o=test',
                                    _ldap.SCOPE_ONELEVEL, 
                                    schema.getSearchFilter(), None, 
                                    schema ).resultsList
        self.failUnless( len(searchLister) == 1 )
        self.failUnless(searchLister[0]['uid'][0].lower() == 'abogus')
        self.failUnless(searchLister[0]['distinguishedName'][0] == 'uid=abogus,ou=ST ScI,o=test')

    def test08(self):
        """ 08 testSearchLister: searching with people in OUs by OU"""
        schema = self.directory.getPersonSchema()
        schema.setPersonsOUAttr('dn')
        searchLister = SearchLister(self.user, 'ou=ST ScI,o=test', 'uid', 
                                    'person', 'ou=ST ScI,o=test',
                                    _ldap.SCOPE_ONELEVEL, 
                                    schema.getSearchFilter(), ['uid'], 
                                    schema ).resultsList
        self.failUnless( len(searchLister) == 1 )
        self.failUnless(searchLister[0]['uid'][0].lower() == 'abogus')
        self.failUnless(searchLister[0]['distinguishedName'][0] == 'uid=abogus,ou=ST ScI,o=test')

    def tearDown(self):
        self.user.unbind()

if __name__ == '__main__':
    
    tests = unittest.makeSuite(testSearchLister, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
